#ifndef CLIENTCC_H
#define CLIENTCC_H

#include <string>
#include <vector>
#include <new>

enum kind     { SUBJ = 0, RES, ACT };
enum att_type { TIME = 0, STRING, BOOL, INT, DATE, URI };
enum answer   { DENY = 0, PERMIT, INDET, NOTA };

class Attribute {
private:
  Attribute();
public:
  ~Attribute();
  Attribute(const std::string&, const std::string&, att_type);
  Attribute(const Attribute&);
  Attribute &operator=(const Attribute&);
  std::string Name()  const  { return name; }
  std::string Value() const  { return value; }
  att_type Type()     const  { return type; }
private:
  std::string name;
  std::string value;
  att_type type;
};

typedef std::vector<Attribute> Attributes;

class Obligation {
private:
  Obligation();
 public:
  ~Obligation();
  Obligation(const std::string&, const Attributes&, answer);
  Obligation(const Obligation &);
  Obligation &operator=(const Obligation &);
  std::string GetId()   const { return id; }
  Attributes GetArgs()  const { return args; }
  answer GetCondition() const { return condition; }
private:
  Attributes args;
  std::string id;
  answer condition;
};
 
typedef std::vector<Obligation> Obligations;

class EvalResult {
private:
  EvalResult();
public:
  EvalResult(answer ans, Obligations& obs, std::string name);
  answer GetResult();
  Obligations GetObligations();
  std::string GetId();
private:
  Obligations o;
  answer c;
  std::string n;
};

typedef std::vector<EvalResult> EvalResults;

class no_conn : public std::exception {};

class PEPClient;

class Connection {
  friend class PEPClient;
public:
  Connection(const std::string&, int, const std::string&, bool);
  ~Connection();
private:
  void *h;
};

class PEPClient {
 public:
  PEPClient(const std::string&, const std::string &, const std::string &);
  PEPClient(int v);
private:
  PEPClient(PEPClient &p);
  PEPClient &operator=(const PEPClient &pc);
public:
  ~PEPClient();
  bool Init(int);
  bool SetAttr(Attributes &, kind);
  void Attach(Connection *);
  bool Send(const std::string &, int, int, int, EvalResults &);
private:
  Connection *c;
  void *h;
};

extern answer int2ans(int v);
extern int ans2int(answer a);
#endif
